#! /bin/sh
#
# Copyright 2003 Gatekeeper Systems Inc.
# All rights reserved
#

# Things to consider...
# a. caller will need to gaurd against other running instances of this script
# b. this script will need to be called under the following circumstances:
#         i. when the configuration is loaded during boot, to update the host/fqdn name with the busID
#        ii. when a configuration is manually loaded via the front panel, as above
#       iii. when the busID is altered via the front panel, as above
#        iv. at regular intervals (within the polling loop?!?), to update changing IP/domain names as the vehicle moves between wireless networks

USAGE="gkgun-refresh-dhcp interface [hostname]"
LOCK=/tmp/refresh.dhcp
RETVAL=0

cleanup() {
    rm -f ${LOCK}
}

trap "cleanup ; exit 0" HUP TERM INT QUIT

IFACE="$1"

if [ -z "${IFACE}" ]; then
    echo "$0: no interface specified, terminating." 1>&2
    echo ${USAGE} 1>&2
    exit 0
fi

# if the lock file is already present, terminate immediately
if [ -e "${LOCK}" ] ; then
    exit 1
fi

touch ${LOCK}

ifconfig ${IFACE} >/dev/null
STAT=$?
if [ ${STAT} -ne 0 ] ; then
    # invalid interface?!?
    echo "$0: ifconfig returned ${STAT} for ${IFACE}, exiting." 1>&2
    # no point in retrying, do not return an error...
else
    # create a cache for the domain name, for comparison purposes...
    touch /etc/domainname

    # take a reference to the current network info...
    HOST="$2"
    if [ -z "${HOST}" ]; then
        # no hostname supplied, use what we already have...
        HOST=`cat /etc/hostname`
    else
        # update the hostname to the one supplied...
        echo ${HOST} >/etc/hostname
        hostname -F /etc/hostname
    fi

    DOMAIN=`cat /etc/domainname`
    if [ -z ${DOMAIN} ]; then
        # only set the hostname if there is no cached domain name...
        dns="-H ${HOST}"
    else
        # set the fully qualified domain name if there is a cached domain name...
        dns="-F ${HOST}.${DOMAIN}"
    fi
    
    echo "$0: querying DHCP server for ${IFACE} using ${dns}" 1>&2
    
    # renew DHCP request, refresh lease and get any new network info...
    udhcpc -n -q -f -i ${IFACE} ${dns} >/dev/null 2>&1
    STAT=$?
    
    if [ ${STAT} -ne 0 ] ; then
        # take the interface down on errors...
        echo "$0: initial dhcp request returned ${STAT}, not enabling ${IFACE}" 1>&2
        ifconfig ${IFACE} down

        # TODO: if we were so inclined as to support APIPA, here may be an apporpriate place to handle it...

        # dont bother retrying, the DHCP client will have already, do not retrun an error
    else
        ifconfig ${IFACE} 1>&2
        
        NEWDOMAIN=`cat /etc/resolv.conf | sed -ne 's%.*domain \([.]*\)%\1%p'`
        if [ X${DOMAIN} != X${NEWDOMAIN} ]; then
            # touch up the domainname on disk...
            echo ${NEWDOMAIN} >/etc/domainname
            # force the caller to try again...
            RETVAL=1
        fi

        PROCESSES="/tmp/ps.out"
        ps -ef > ${PROCESSES}

        HOOK="/fat/hooks/post-dhcp"
        ACTIVE=$(cat ${PROCESSES} | egrep -c ${HOOK})
        # call the hook script if there is one avail. and it isnt already running...
        if [ -x "${HOOK}" -a ${ACTIVE} -eq 0 ]; then
            # pass along the interface, host and domain names...
            HOST=`cat /etc/hostname`
            DOMAIN=`cat /etc/domainname`
            ${HOOK} ${IFACE} ${HOST} ${DOMAIN} &
        fi
    fi
fi

cleanup

exit ${RETVAL}

